@@ -80,11 +80,9 @@ void LanguageManager::generateLangList()
 	langList.insert("et",       langPair("Estonian",            QObject::tr( "Estonian" )) );
 	langList.insert("de",       langPair("German",              QObject::tr( "German" )) );
 	langList.insert("de_CH",    langPair("German (Swiss)",      QObject::tr( "German (Swiss)" )) );
-	langList.insert("de_ol",    langPair("German (Trad.)",      QObject::tr( "German (Trad.)" )) );
+	langList.insert("de_1901",  langPair("German (Trad.)",      QObject::tr( "German (Trad.)" )) );
 	langList.insert("fi",       langPair("Finnish",             QObject::tr( "Finnish" )) );
 	langList.insert("fr",       langPair("French",              QObject::tr( "French" )) );
-	langList.insert("ga",       langPair("Galician",            QObject::tr( "Galician" )) );
-	langList.insert("galician", langPair("Galician",            QObject::tr( "Galician" )) );
 	langList.insert("gl",       langPair("Galician",            QObject::tr( "Galician" )) );
 	langList.insert("el",       langPair("Greek",               QObject::tr( "Greek" )) );
 	langList.insert("he",       langPair("Hebrew",              QObject::tr( "Hebrew" )) );
@@ -96,6 +94,7 @@ void LanguageManager::generateLangList()
 	langList.insert("ja",       langPair("Japanese",            QObject::tr( "Japanese" )) );
 	langList.insert("km",       langPair("Khmer",               QObject::tr( "Khmer" )) );
 	langList.insert("ko",       langPair("Korean",              QObject::tr( "Korean" )) );
+	langList.insert("ku",       langPair("Kurdish",             QObject::tr( "Kurdish" )) );
 	langList.insert("lo",       langPair("Lao",                 QObject::tr( "Lao" )) );
 	langList.insert("la",       langPair("Latin",               QObject::tr( "Latin" )) );
 	langList.insert("lt",       langPair("Lithuanian",          QObject::tr( "Lithuanian" )) );
@@ -314,7 +313,7 @@ const QString LanguageManager::getHyphFilename(const QString & lang, bool langIs
 {
 	if(langIsAbbreviated)
 		return hyphLangList.value(lang);
-	return hyphLangList.value(getAbbrevFromLang(lang, false));
+	return hyphLangList.value(getAbbrevFromLang(lang, false, false));
 }
 
 const QStringList LanguageManager::hyphLangs()
