@@ -238,6 +238,7 @@ public:
 	QMap<QString, QPointer<ScrAction> > scrActions;
 	QMap<QString, QPointer<ScrAction> > scrRecentFileActions;
 	QMap<QString, QPointer<ScrAction> > scrWindowsActions;
+	QMap<QString, QPointer<ScrAction> > scrScrapActions;
 	QMap<QString, QPointer<ScrAction> > scrLayersActions;
 	QMap<QString, QPointer<ScrAction> > scrRecentPasteActions;
 	QMap<QString, QPointer<ScToolBar> > scrToolBars;
@@ -264,6 +265,7 @@ public slots:
 	//void setItemFillTransparency(double t);
 	//void setItemLineTransparency(double t);
 	void setStatusBarMousePosition(double xp, double yp);
+	void setStatusBarTextPosition(double base, double yp);
 	void setStatusBarInfoText(QString newText);
 	bool DoFileClose();
 	//bool DoSaveClose();
@@ -280,7 +282,7 @@ public slots:
 	//void doDeHyphenate();
 	void slotTest();
 	void slotTest2();
-	void PutScrap();
+	void PutScrap(int scID);
 	void PutToPatterns();
 	void changeLayer(int);
 	void showLayer();
@@ -303,7 +305,8 @@ public slots:
 	*/
 	void slotFileAppend();
 
-	void removeRecent(QString fn);
+	void removeRecent(QString fn, bool fromFileWatcher = false);
+	void removeRecentFromWatcher(QString fn);
 	void loadRecent(QString fn);
 	void rebuildRecentFileMenu();
 	void rebuildRecentPasteMenu();
