@@ -5,26 +5,34 @@ a copyright and/or license notice that predates the release of Scribus 1.3.2
 for which a new license (GPL+exception) is in place.
 */
 #include "tabruler.h"
-#include "tabruler.moc"
-#include <qvariant.h>
-#include <qcombobox.h>
-#include <qlabel.h>
-#include <qpushbutton.h>
-#include "mspinbox.h"
-#include <qtoolbutton.h>
-#include <qlayout.h>
-#include <qtooltip.h>
-#include <qwhatsthis.h>
-#include <qpainter.h>
-#include <qcursor.h>
-#include <qcolor.h>
+#include <QVariant>
+#include <QComboBox>
+#include <QLabel>
+#include <QPushButton>
+#include <QToolButton>
+#include <QLayout>
+#include <QToolTip>
+#include <QPainter>
+#include <QCursor>
+#include <QColor>
+#include <QApplication>
+#include <QPolygon>
+#include <QPixmap>
+#include <QMouseEvent>
+#include <QEvent>
+#include <QPaintEvent>
+#include "commonstrings.h"
 #include "units.h"
 #include "scribusstructs.h"
-extern QPixmap loadIcon(QString nam);
+#include "scrspinbox.h"
+#include "util_icon.h"
+#include "util.h"
 
-RulerT::RulerT(QWidget *pa, int ein, QValueList<PageItem::TabRecord> Tabs, bool ind, double wid) : QWidget(pa)
+RulerT::RulerT(QWidget *pa, int ein, QList<ParagraphStyle::TabRecord> Tabs, bool ind, double wid) : QWidget(pa)
 {
-	setEraseColor(QColor(255,255,255));
+	QPalette palette;
+	palette.setColor(backgroundRole(), QColor(255,255,255));
+	setPalette(palette);
 	unitIndex = ein;
 	iter=unitRulerGetIter1FromIndex(unitIndex);
 	iter2=unitRulerGetIter2FromIndex(unitIndex);
@@ -48,12 +56,22 @@ RulerT::RulerT(QWidget *pa, int ein, QValueList<PageItem::TabRecord> Tabs, bool
 	else
 	{
 		Width = wid;
-		setMinimumSize(QSize(QMIN(static_cast<int>(Width), 400),25));
+		setMinimumSize(QSize(qMin(static_cast<int>(Width), 400),25));
 		setMaximumSize(QSize(4000,25));
-		resize(QMIN(static_cast<int>(wid), 400), 25);
+		resize(qMin(static_cast<int>(wid), 400), 25);
 	}
 }
 
+void RulerT::setTabs(QList<ParagraphStyle::TabRecord> Tabs, int dEin)
+{
+	unitIndex = dEin;
+	iter  = unitRulerGetIter1FromIndex(unitIndex);
+	iter2 = unitRulerGetIter2FromIndex(unitIndex);
+	tabValues = Tabs;
+	actTab    = -1;
+	repaint();
+}
+
 void RulerT::paintEvent(QPaintEvent *)
 {
 	double xl;
@@ -61,9 +79,9 @@ void RulerT::paintEvent(QPaintEvent *)
 	p.begin(this);
 	p.drawLine(0, 24, width(), 24);
 	p.translate(-offset, 0);
-	p.setBrush(black);
+	p.setBrush(Qt::black);
 	p.setFont(font());
-	p.setPen(QPen(black, 1, SolidLine, FlatCap, MiterJoin));
+	p.setPen(QPen(Qt::black, 1, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin));
 	for (xl = 0; xl < width()+offset; xl += iter)
 	{
 		if (xl < offset)
@@ -106,9 +124,9 @@ void RulerT::paintEvent(QPaintEvent *)
 		for (int yg = 0; yg < static_cast<int>(tabValues.count()); yg++)
 		{
 			if (yg == actTab)
-				p.setPen(QPen(red, 2, SolidLine, FlatCap, MiterJoin));
+				p.setPen(QPen(Qt::red, 2, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin));
 			else
-				p.setPen(QPen(black, 2, SolidLine, FlatCap, MiterJoin));
+				p.setPen(QPen(Qt::black, 2, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin));
 			switch (static_cast<int>(tabValues[yg].tabType))
 			{
 				case 0:
@@ -136,12 +154,12 @@ void RulerT::paintEvent(QPaintEvent *)
 	}
 	if (haveInd)
 	{
-		p.setPen(QPen(blue, 1, SolidLine, FlatCap, MiterJoin));
-		p.setBrush(blue);
-		QPointArray cr;
+		p.setPen(QPen(Qt::blue, 1, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin));
+		p.setBrush(Qt::blue);
+		QPolygon cr;
 		cr.setPoints(3, qRound(firstLine+leftIndent), 12, qRound(firstLine+leftIndent-4), 0, qRound(firstLine+leftIndent+4), 0);
 		p.drawPolygon(cr);
-		QPointArray cr2;
+		QPolygon cr2;
 		cr2.setPoints(3, qRound(leftIndent), 12, qRound(leftIndent+4), 24, qRound(leftIndent-4), 24);
 		p.drawPolygon(cr2);
 	}
@@ -189,9 +207,9 @@ void RulerT::mousePressEvent(QMouseEvent *m)
 			}
 		}
 	}
-	if ((rulerCode == 0) && (m->button() == LeftButton))
+	if ((rulerCode == 0) && (m->button() == Qt::LeftButton))
 	{
-		struct PageItem::TabRecord tb;
+		ParagraphStyle::TabRecord tb;
 		tb.tabPosition = static_cast<double>(m->x() + offset);
 		tb.tabType = 0;
 		tb.tabFillChar = QChar();
@@ -204,7 +222,7 @@ void RulerT::mousePressEvent(QMouseEvent *m)
 		emit typeChanged(tabValues[actTab].tabType);
 		emit tabMoved(tabValues[actTab].tabPosition);
 		emit fillCharChanged(tabValues[actTab].tabFillChar);
-		qApp->setOverrideCursor(QCursor(SizeHorCursor), true);
+		qApp->changeOverrideCursor(QCursor(Qt::SizeHorCursor));
 	}
 	mouseX = m->x();
 }
@@ -216,7 +234,7 @@ void RulerT::mouseReleaseEvent(QMouseEvent *m)
 	{
 		if (rulerCode == 3)
 		{
-			if (m->button() == RightButton)
+			if (m->button() == Qt::RightButton)
 			{
 				tabValues[actTab].tabType += 1;
 				if (tabValues[actTab].tabType > 4)
@@ -230,9 +248,7 @@ void RulerT::mouseReleaseEvent(QMouseEvent *m)
 	{
 		if (rulerCode == 3)
 		{
-			QValueList<PageItem::TabRecord>::Iterator it;
-			it = tabValues.at(actTab);
-			tabValues.remove(it);
+			tabValues.removeAt(actTab);
 			actTab = 0;
 			if (tabValues.count() != 0)
 			{
@@ -243,7 +259,7 @@ void RulerT::mouseReleaseEvent(QMouseEvent *m)
 			else
 				emit noTabs();
 			repaint();
-			qApp->setOverrideCursor(QCursor(ArrowCursor), true);
+			qApp->changeOverrideCursor(QCursor(Qt::ArrowCursor));
 		}
 	}
 	rulerCode = 0;
@@ -256,7 +272,7 @@ void RulerT::mouseMoveEvent(QMouseEvent *m)
 	QRect fpo;
 	if ((mousePressed) && (m->y() < height()) && (m->y() > 0) && (m->x() > 0) && (m->x() < width()))
 	{
-		qApp->setOverrideCursor(QCursor(SizeHorCursor), true);
+		qApp->changeOverrideCursor(QCursor(Qt::SizeHorCursor));
 		switch (rulerCode)
 		{
 			case 1:
@@ -298,19 +314,19 @@ void RulerT::mouseMoveEvent(QMouseEvent *m)
 	}
 	if ((!mousePressed) && (m->y() < height()) && (m->y() > 0) && (m->x() > 0) && (m->x() < width()))
 	{
-		qApp->setOverrideCursor(QCursor(loadIcon("tab.png"), 3), true);
+		qApp->changeOverrideCursor(QCursor(loadIcon("tab.png"), 3));
 		if (haveInd)
 		{
 			fpo = QRect(static_cast<int>(firstLine+leftIndent-offset)-4, 0, 8, 12);
 			if (fpo.contains(m->pos()))
 			{
-				qApp->setOverrideCursor(QCursor(SizeHorCursor), true);
+				qApp->changeOverrideCursor(QCursor(Qt::SizeHorCursor));
 				return;
 			}
 			fpo = QRect(static_cast<int>(leftIndent-offset)-4, 12, 8, 12);
 			if (fpo.contains(m->pos()))
 			{
-				qApp->setOverrideCursor(QCursor(SizeHorCursor), true);
+				qApp->changeOverrideCursor(QCursor(Qt::SizeHorCursor));
 				return;
 			}
 		}
@@ -321,7 +337,7 @@ void RulerT::mouseMoveEvent(QMouseEvent *m)
 				fpo = QRect(static_cast<int>(tabValues[yg].tabPosition-offset)-3, 15, 8, 8);
 				if (fpo.contains(m->pos()))
 				{
-					qApp->setOverrideCursor(QCursor(SizeHorCursor), true);
+					qApp->changeOverrideCursor(QCursor(Qt::SizeHorCursor));
 					return;
 				}
 			}
@@ -332,26 +348,21 @@ void RulerT::mouseMoveEvent(QMouseEvent *m)
 void RulerT::leaveEvent(QEvent*)
 {
 	if ((mousePressed) && (rulerCode == 3))
-	{
-		qApp->setOverrideCursor(QCursor(loadIcon("DelPoint.png"), 1, 1), true);
-		return;
-	}
+		qApp->changeOverrideCursor(QCursor(loadIcon("DelPoint.png"), 1, 1));
 	else
-		qApp->setOverrideCursor(QCursor(ArrowCursor), true);
+		qApp->changeOverrideCursor(QCursor(Qt::ArrowCursor));
 }
 
 void RulerT::updateTabList()
 {
-	struct PageItem::TabRecord tb;
+	ParagraphStyle::TabRecord tb;
 	tb.tabPosition = tabValues[actTab].tabPosition;
 	tb.tabType = tabValues[actTab].tabType;
 	tb.tabFillChar =  tabValues[actTab].tabFillChar;
-	QValueList<PageItem::TabRecord>::Iterator it;
 	int gg = static_cast<int>(tabValues.count()-1);
 	int g = gg;
-	it = tabValues.at(actTab);
-	tabValues.remove(it);
-	for (int yg = static_cast<int>(tabValues.count()); yg > -1; yg--)
+	tabValues.removeAt(actTab);
+	for (int yg = static_cast<int>(tabValues.count()-1); yg > -1; yg--)
 	{
 		if (tb.tabPosition < tabValues[yg].tabPosition)
 			g = yg;
@@ -364,8 +375,7 @@ void RulerT::updateTabList()
 	}
 	else
 	{
-		it = tabValues.at(actTab);
-		tabValues.insert(it, tb);
+		tabValues.insert(actTab, tb);
 	}
 }
 
@@ -398,18 +408,24 @@ void RulerT::decreaseOffset()
 
 void RulerT::changeTab(int t)
 {
+	if (actTab < 0 || actTab >= tabValues.count())
+		return;
 	tabValues[actTab].tabType = t;
 	repaint();
 }
 
 void RulerT::changeTabChar(QChar t)
 {
+	if (actTab < 0 || actTab >= tabValues.count())
+		return;
 	tabValues[actTab].tabFillChar = t;
 	repaint();
 }
 
 void RulerT::moveTab(double t)
 {
+	if (actTab < 0 || actTab >= tabValues.count())
+		return;
 	tabValues[actTab].tabPosition = t;
 	updateTabList();
 	repaint();
@@ -445,78 +461,114 @@ void RulerT::moveLeftIndent(double t)
 	repaint();
 }
 
-Tabruler::Tabruler( QWidget* parent, bool haveFirst, int dEin, QValueList<PageItem::TabRecord> Tabs, double wid ) : QWidget( parent )
+Tabruler::Tabruler( QWidget* parent, bool haveFirst, int dEin, QList<ParagraphStyle::TabRecord> Tabs, double wid ) : QWidget( parent )
 {
 	docUnitRatio=unitGetRatioFromIndex(dEin);
 	double ww;
 	ww = (wid < 0) ? 4000 : wid;
-	setName( "tabruler" );
-	tabrulerLayout = new QVBoxLayout( this, 0, 6, "tabrulerLayout");
-	layout2 = new QHBoxLayout( 0, 0, 6, "layout2");
-	TypeCombo = new QComboBox( false, this, "TypeCombo" );
-	TypeCombo->clear();
-	TypeCombo->insertItem( tr( "Left" ) );
-	TypeCombo->insertItem( tr( "Right" ) );
-	TypeCombo->insertItem( tr( "Full Stop" ) );
-	TypeCombo->insertItem( tr( "Comma" ) );
-	TypeCombo->insertItem( tr( "Center" ) );
-	layout2->addWidget( TypeCombo );
-	rulerScrollL = new QToolButton( LeftArrow, this, "rulerScrollL" );
+	tabrulerLayout = new QVBoxLayout( this );
+	tabrulerLayout->setMargin(0);
+	tabrulerLayout->setSpacing(5);
+	layout2 = new QHBoxLayout;
+	layout2->setMargin(0);
+	layout2->setSpacing(5);
+
+	rulerScrollL = new QToolButton(this);
+	rulerScrollL->setArrowType(Qt::LeftArrow);
 	rulerScrollL->setAutoRepeat( true );
 	layout2->addWidget( rulerScrollL );
 	ruler = new RulerT( this, dEin, Tabs, haveFirst, wid );
-	ruler->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)5, 0, 0, ruler->sizePolicy().hasHeightForWidth() ) );
+	ruler->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
 	layout2->addWidget( ruler );
-	rulerScrollR = new QToolButton( RightArrow, this, "RulserScrollR" );
+	rulerScrollR = new QToolButton(this);
+	rulerScrollR->setArrowType(Qt::RightArrow);
 	rulerScrollR->setAutoRepeat( true );
 	layout2->addWidget( rulerScrollR );
-	tabrulerLayout->addLayout( layout2 );
-	layout1 = new QHBoxLayout( 0, 0, 6, "layout1" );
+
+	layout1 = new QHBoxLayout;
+	layout1->setMargin(0);
+	layout1->setSpacing(5);
 	layout1->setAlignment( Qt::AlignTop );
-	tabData = new MSpinBox( 0, ww / docUnitRatio, this, 1 );
+	typeCombo = new QComboBox(this);
+	typeCombo->setEditable(false);
+	typeCombo->clear();
+	typeCombo->addItem( tr( "Left" ) );
+	typeCombo->addItem( tr( "Right" ) );
+	typeCombo->addItem( tr( "Period" ) );
+	typeCombo->addItem( tr( "Comma" ) );
+	typeCombo->addItem( tr( "Center" ) );
+	layout1->addWidget( typeCombo );
+	tabData = new ScrSpinBox( 0, ww / docUnitRatio, this, dEin );
 	tabData->setValue(0);
-	positionLabel = new QLabel( tabData, tr("&Position:"), this, "positionLabel" );
+	positionLabel = new QLabel( tr("&Position:"), this );
+	positionLabel->setBuddy(tabData);
 	layout1->addWidget( positionLabel );
 	layout1->addWidget( tabData );
-	tabFillCombo = new QComboBox( true, this, "tabFillCombo" );
+	tabFillCombo = new QComboBox(this);
 	tabFillCombo->setEditable(false);
-	tabFillCombo->insertItem( tr("None"), "tab fill");
-	tabFillCombo->insertItem( tr("Dot"));
-	tabFillCombo->insertItem( tr("Hyphen"));
-	tabFillCombo->insertItem( tr("Underscore"));
-	tabFillCombo->insertItem( tr("Custom"));
-	tabFillComboT = new QLabel(tabFillCombo, tr( "Fill Char:" ), this, "tabFillComboT" );
+	tabFillCombo->addItem( tr("None", "tab fill"));
+	tabFillCombo->addItem( tr("Dot"));
+	tabFillCombo->addItem( tr("Hyphen"));
+	tabFillCombo->addItem( tr("Underscore"));
+	tabFillCombo->addItem( tr("Custom"));
+	tabFillComboT = new QLabel( tr( "Fill Char:" ), this );
+	tabFillComboT->setBuddy(tabFillCombo);
 	layout1->addWidget( tabFillComboT );
 	layout1->addWidget( tabFillCombo );
+
+	layout4 = new QHBoxLayout;
+	layout4->setMargin(0);
+	layout4->setSpacing(5);
+
+	indentLayout = new QHBoxLayout;
+	indentLayout->setMargin(0);
+	indentLayout->setSpacing(5);
 	if (haveFirst)
 	{
-		firstLineData = new MSpinBox( -3000, ww / docUnitRatio, this, 1);
+		firstLineData = new ScrSpinBox( -3000, ww / docUnitRatio, this, dEin);
 		firstLineData->setValue(0);
-		firstLineLabel = new QLabel( "", this, "firstLineLabel" );
+		firstLineLabel = new QLabel(this);
 		firstLineLabel->setText("");
 		firstLineLabel->setPixmap(loadIcon("firstline.png"));
-		layout1->addWidget( firstLineLabel );
-		layout1->addWidget( firstLineData );
-		leftIndentData = new MSpinBox( 0, ww / docUnitRatio, this, 1 );
+		indentLayout->addWidget( firstLineLabel );
+		indentLayout->addWidget( firstLineData );
+		leftIndentData = new ScrSpinBox( 0, ww / docUnitRatio, this, dEin );
 		leftIndentData->setValue(0);
-		leftIndentLabel = new QLabel( "", this, "leftIndentLabel" );
+		leftIndentLabel = new QLabel(this);
 		leftIndentLabel->setText("");
 		leftIndentLabel->setPixmap(loadIcon("leftindent.png"));
-		layout1->addWidget( leftIndentLabel );
-		layout1->addWidget( leftIndentData );
+		layout4->addWidget( leftIndentLabel );
+		layout4->addWidget( leftIndentData );
+		layout4->addStretch(10);
+		rightIndentLabel = new QLabel(this);
+		rightIndentLabel->setText("");
+		rightIndentLabel->setPixmap(loadIcon("rightindent.png"));
+		rightIndentData = new ScrSpinBox(0, ww / docUnitRatio, this, dEin);
+		rightIndentData->setValue(0);
+		indentLayout->addWidget(rightIndentLabel);
+		indentLayout->addWidget(rightIndentData);
 	}
-	clearButton = new QPushButton( this, "clearButton" );
+	clearButton = new QPushButton( this );
 	clearButton->setText( tr( "Delete All" ) );
-	layout1->addWidget( clearButton );
+	indentLayout->addSpacing(20);
+	indentLayout->addWidget( clearButton);
+	indentLayout->addStretch(10);
 	if (!haveFirst)
 	{
 		QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
 		layout1->addItem( spacer );
 	}
+	
+	layout1->addStretch( 10 );
 	tabrulerLayout->addLayout( layout1 );
-	TypeCombo->setEnabled(false);
+	tabrulerLayout->addLayout( layout2 );
+	indentLayout->addStretch( 10 );
+	tabrulerLayout->addLayout( indentLayout );
+	tabrulerLayout->addLayout(layout4);
+
 	tabData->setEnabled(false);
 	tabFillCombo->setEnabled(false);
+	typeCombo->setEnabled(false);
 	if (Tabs.count() == 0)
 		clearButton->setEnabled(false);
 	resize( minimumSizeHint() );
@@ -526,45 +578,76 @@ Tabruler::Tabruler( QWidget* parent, bool haveFirst, int dEin, QValueList<PageIt
 	connect(rulerScrollR, SIGNAL(released()), this, SLOT(resetOFfR()));
 	connect(ruler, SIGNAL(typeChanged(int)) , this, SLOT(setTabType(int)));
 	connect(ruler, SIGNAL(fillCharChanged(QChar)) , this, SLOT(setTabFillChar(QChar)));
-	connect(TypeCombo, SIGNAL(activated(int)), this, SLOT(setType()));
+	connect(typeCombo, SIGNAL(activated(int)), this, SLOT(setType()));
 	connect(tabFillCombo, SIGNAL(activated(int)), this, SLOT(setFillChar()));
-	connect(tabFillCombo, SIGNAL(textChanged(const QString &)), this, SLOT(setCustomFillChar(const QString &)));
+	connect(tabFillCombo, SIGNAL(editTextChanged(const QString &)), this, SLOT(setCustomFillChar(const QString &)));
 	connect(ruler, SIGNAL(tabMoved(double)) , this, SLOT(setTabData(double)));
 	connect(ruler, SIGNAL(tabSelected()), this, SLOT(tabAdded()));
 	connect(ruler, SIGNAL(newTab()), this, SLOT(tabAdded()));
 	connect(ruler, SIGNAL(noTabs()), this, SLOT(lastTabRemoved()));
-	connect(tabData, SIGNAL(valueChanged(int)), this, SLOT(setTab()));
+	connect(tabData, SIGNAL(valueChanged(double)), this, SLOT(setTab()));
 	connect(clearButton, SIGNAL(clicked()), this, SLOT(clearAll()));
+
+	tabFillCombo->setToolTip( tr( "Fill Character of Tab" ) );
+	typeCombo->setToolTip( tr( "Type/Orientation of Tab" ) );
+	tabData->setToolTip( tr( "Position of Tab" ) );
+
 	if (haveFirst)
 	{
 		connect(ruler, SIGNAL(firstLineMoved(double)) , this, SLOT(setFirstLineData(double)));
 		connect(ruler, SIGNAL(leftIndentMoved(double)) , this, SLOT(setLeftIndentData(double)));
 		connect(ruler, SIGNAL(mouseReleased()), this, SIGNAL(tabrulerChanged()));
-		connect(firstLineData, SIGNAL(valueChanged(int)), this, SLOT(setFirstLine()));
-		connect(leftIndentData, SIGNAL(valueChanged(int)), this, SLOT(setLeftIndent()));
-		QToolTip::add( firstLineData, tr( "Indentation for first line of the paragraph" ) );
-		QToolTip::add( leftIndentData, tr( "Indentation from the left for the whole paragraph" ) );
+		connect(ruler, SIGNAL(mouseReleased()), this, SLOT(slotMouseReleased()));
+		connect(firstLineData, SIGNAL(valueChanged(double)), this, SLOT(setFirstLine()));
+		connect(leftIndentData, SIGNAL(valueChanged(double)), this, SLOT(setLeftIndent()));
+		connect(rightIndentData, SIGNAL(valueChanged(double)), this, SLOT(setRightIndent()));
+		firstLineData->setToolTip( tr( "Indentation for first line of the paragraph" ) );
+		leftIndentData->setToolTip( tr( "Indentation from the left for the whole paragraph" ) );
+		rightIndentData->setToolTip( tr( "Indentation from the right for the whole paragraph" ) );
+		firstLineLabel->setToolTip(firstLineData->toolTip());
+		leftIndentLabel->setToolTip(leftIndentData->toolTip());
+		rightIndentLabel->setToolTip(rightIndentData->toolTip());
 	}
-	QToolTip::add( clearButton, tr( "Delete all Tabulators" ) );
+	clearButton->setToolTip( tr( "Delete all Tabulators" ) );
 	QString ein = unitGetSuffixFromIndex(dEin);
 	if (dEin == 2)
 	{
 		if (haveFirst)
 		{
-			firstLineData->setDecimals(10000);
-			leftIndentData->setDecimals(10000);
+			firstLineData->setDecimals(4);
+			leftIndentData->setDecimals(4);
+			rightIndentData->setDecimals(4);
 		}
-		tabData->setDecimals(10000);
+		tabData->setDecimals(4);
 	}
 	if (haveFirst)
 	{
 		firstLineData->setSuffix(ein);
 		leftIndentData->setSuffix(ein);
+		rightIndentData->setSuffix(ein);
 	}
 	tabData->setSuffix(ein);
 	haveF = haveFirst;
 }
 
+void Tabruler::setTabs(QList<ParagraphStyle::TabRecord> Tabs, int dEin)
+{
+	docUnitRatio=unitGetRatioFromIndex(dEin);
+	tabData->setNewUnit(dEin);
+	if (haveF)
+	{
+		firstLineData->setNewUnit(dEin);
+		leftIndentData->setNewUnit(dEin);
+		rightIndentData->setNewUnit(dEin);
+	}
+	ruler->setTabs(Tabs, dEin);
+	if (Tabs.count() == 0)
+		clearButton->setEnabled(false);
+	tabData->setEnabled(false);
+	tabFillCombo->setEnabled(false);
+	typeCombo->setEnabled(false);
+}
+
 void Tabruler::resetOFfL()
 {
 	if (!rulerScrollL->isDown())
@@ -583,32 +666,34 @@ void Tabruler::clearAll()
 	ruler->repaint();
 	lastTabRemoved();
 	emit tabrulerChanged();
+	emit tabsChanged();
 }
 
 void Tabruler::tabAdded()
 {
-	TypeCombo->setEnabled(true);
+	typeCombo->setEnabled(true);
 	tabData->setEnabled(true);
 	clearButton->setEnabled(true);
 	tabFillCombo->setEnabled(true);
 	emit tabrulerChanged();
+	emit tabsChanged();
 }
 
 void Tabruler::lastTabRemoved()
 {
-	TypeCombo->setEnabled(false);
+	typeCombo->setEnabled(false);
 	tabData->setEnabled(false);
 	clearButton->setEnabled(false);
 	tabFillCombo->setEnabled(false);
 	emit tabrulerChanged();
+	emit tabsChanged();
 }
 
 void Tabruler::setFillChar()
 {
-	disconnect(tabFillCombo, SIGNAL(textChanged(const QString &)), this, SLOT(setCustomFillChar(const QString &)));
-	disconnect(tabFillCombo, SIGNAL(activated(int)), this, SLOT(setFillChar()));
+	tabFillCombo->blockSignals(true);
 	QChar ret;
-	switch (tabFillCombo->currentItem())
+	switch (tabFillCombo->currentIndex())
 	{
 		case 0:
 			tabFillCombo->setEditable(false);
@@ -628,25 +713,26 @@ void Tabruler::setFillChar()
 			break;
 		case 4:
 			tabFillCombo->setEditable(true);
-			tabFillCombo->setEditText( tr("Custom:"));
+			setCurrentComboItem(tabFillCombo, CommonStrings::trCustomTabFill);
 			break;
 	}
-	if (tabFillCombo->currentItem() != 4)
+	if (tabFillCombo->currentIndex() != 4)
 		ruler->changeTabChar(ret);
-	connect(tabFillCombo, SIGNAL(activated(int)), this, SLOT(setFillChar()));
-	connect(tabFillCombo, SIGNAL(textChanged(const QString &)), this, SLOT(setCustomFillChar(const QString &)));
+	tabFillCombo->blockSignals(false);
+	emit tabrulerChanged();
+	emit tabsChanged();
 }
 
 void Tabruler::setCustomFillChar(const QString &txt)
 {
-	if (txt == tr("Custom:"))
+	if (txt == CommonStrings::trCustomTabFill)
 		return;
-	disconnect(tabFillCombo, SIGNAL(textChanged(const QString &)), this, SLOT(setCustomFillChar(const QString &)));
-	disconnect(tabFillCombo, SIGNAL(activated(int)), this, SLOT(setFillChar()));
-	QChar ret = txt[txt.length()-1];
+	tabFillCombo->blockSignals(true);
+	QChar ret = (txt.length() > 0) ? txt[txt.length()-1] : QChar::Null;
 	ruler->changeTabChar(ret);
-	connect(tabFillCombo, SIGNAL(activated(int)), this, SLOT(setFillChar()));
-	connect(tabFillCombo, SIGNAL(textChanged(const QString &)), this, SLOT(setCustomFillChar(const QString &)));
+	tabFillCombo->blockSignals(false);
+	emit tabrulerChanged();
+	emit tabsChanged();
 }
 
 void Tabruler::setTabFillChar(QChar t)
@@ -654,92 +740,119 @@ void Tabruler::setTabFillChar(QChar t)
 	if (t.isNull())
 	{
 		tabFillCombo->setEditable(false);
-		tabFillCombo->setCurrentItem(0);
+		tabFillCombo->setCurrentIndex(0);
 	}
-	else if (t == ".")
+	else if (t == '.')
 	{
 		tabFillCombo->setEditable(false);
-		tabFillCombo->setCurrentItem(1);
+		tabFillCombo->setCurrentIndex(1);
 	}
-	else if (t == "-")
+	else if (t == '-')
 	{
 		tabFillCombo->setEditable(false);
-		tabFillCombo->setCurrentItem(2);
+		tabFillCombo->setCurrentIndex(2);
 	}
-	else if (t == "_")
+	else if (t == '_')
 	{
 		tabFillCombo->setEditable(false);
-		tabFillCombo->setCurrentItem(3);
+		tabFillCombo->setCurrentIndex(3);
 	}
 	else
 	{
-		tabFillCombo->setCurrentItem(4);
+		tabFillCombo->setCurrentIndex(4);
 		tabFillCombo->setEditable(true);
-		tabFillCombo->setEditText( tr("Custom: ")+QString(t));
+		setCurrentComboItem(tabFillCombo, CommonStrings::trCustomTabFill+QString(t));
 	}
 	emit tabrulerChanged();
+	emit tabsChanged();
 }
 
 void Tabruler::setTabType(int t)
 {
-	TypeCombo->setCurrentItem(t);
+	typeCombo->setCurrentIndex(t);
 	emit tabrulerChanged();
+	emit tabsChanged();
 }
 
 void Tabruler::setType()
 {
-	disconnect(TypeCombo, SIGNAL(activated(int)), this, SLOT(setType()));
-	ruler->changeTab(TypeCombo->currentItem());
-	connect(TypeCombo, SIGNAL(activated(int)), this, SLOT(setType()));
+	typeCombo->blockSignals(true);
+	ruler->changeTab(typeCombo->currentIndex());
+	typeCombo->blockSignals(false);
 	emit tabrulerChanged();
+	emit tabsChanged();
 }
 
 void Tabruler::setTabData(double t)
 {
-	disconnect(tabData, SIGNAL(valueChanged(int)), this, SLOT(setTab()));
+	tabData->blockSignals(true);
 	tabData->setValue(t * docUnitRatio);
-	connect(tabData, SIGNAL(valueChanged(int)), this, SLOT(setTab()));
+	tabData->blockSignals(false);
 	if (!ruler->mousePressed)
+	{
 		emit tabrulerChanged();
+		emit tabsChanged();
+	}
 }
 
 void Tabruler::setTab()
 {
 	ruler->moveTab(tabData->value() / docUnitRatio);
 	emit tabrulerChanged();
+	emit tabsChanged();
 }
 
 void Tabruler::setFirstLineData(double t)
 {
-	disconnect(firstLineData, SIGNAL(valueChanged(int)), this, SLOT(setFirstLine()));
+//	firstLineData->blockSignals(true);
 	firstLineData->setValue(t * docUnitRatio);
-	connect(firstLineData, SIGNAL(valueChanged(int)), this, SLOT(setFirstLine()));
+//	firstLineData->blockSignals(false);
 	if (!ruler->mousePressed)
+	{
 		emit tabrulerChanged();
+		double a, b, value;
+		int c;
+		firstLineData->getValues(&a, &b, &c, &value);
+		emit firstLineChanged(value);
+	}
 }
 
 void Tabruler::setFirstLine()
 {
 	ruler->moveFirstLine(firstLineData->value() / docUnitRatio);
 	emit tabrulerChanged();
+	double a, b, value;
+	int c;
+	firstLineData->getValues(&a, &b, &c, &value);
+	emit firstLineChanged(value);
 }
 
 void Tabruler::setLeftIndentData(double t)
 {
-	disconnect(leftIndentData, SIGNAL(valueChanged(int)), this, SLOT(setLeftIndent()));
+//	leftIndentData->blockSignals(true);
 	leftIndentData->setValue(t * docUnitRatio);
-	connect(leftIndentData, SIGNAL(valueChanged(int)), this, SLOT(setLeftIndent()));
+//	leftIndentData->blockSignals(false);
 	if (!ruler->mousePressed)
+	{
 		emit tabrulerChanged();
+		double a, b, value;
+		int c;
+		leftIndentData->getValues(&a, &b, &c, &value);
+		emit leftIndentChanged(value);
+	}
 }
 
 void Tabruler::setLeftIndent()
 {
 	ruler->moveLeftIndent(leftIndentData->value() / docUnitRatio);
 	emit tabrulerChanged();
+	double a, b, value;
+	int c;
+	leftIndentData->getValues(&a, &b, &c, &value);
+	emit leftIndentChanged(value);
 }
 
-QValueList<PageItem::TabRecord> Tabruler::getTabVals()
+QList<ParagraphStyle::TabRecord> Tabruler::getTabVals()
 {
 	return ruler->tabValues;
 }
@@ -753,3 +866,37 @@ double Tabruler::getLeftIndent()
 {
 	return leftIndentData->value() / docUnitRatio;
 }
+
+void Tabruler::setRightIndentData(double t)
+{
+	rightIndentData->blockSignals(true);
+	rightIndentData->setValue(t * docUnitRatio);
+	rightIndentData->blockSignals(false);
+	if (!ruler->mousePressed)
+	{
+		emit tabrulerChanged();
+		double a, b, value;
+		int c;
+		rightIndentData->getValues(&a, &b, &c, &value);
+		emit rightIndentChanged(value);
+	}
+}
+
+void Tabruler::setRightIndent()
+{
+	emit tabrulerChanged();
+	double a, b, value;
+	int c;
+	rightIndentData->getValues(&a, &b, &c, &value);
+	emit rightIndentChanged(value);
+}
+
+double Tabruler::getRightIndent()
+{
+	return rightIndentData->value() / docUnitRatio;
+}
+
+void Tabruler::slotMouseReleased()
+{
+	emit mouseReleased();
+}
