@@ -634,21 +634,26 @@ void StoryText::applyCharStyle(int pos, uint len, const CharStyle& style )
 	ScText* itText;
 	for (uint i=pos; i < pos+len; ++i) {
 		itText = d->at(i);
-		// #6165 : applying style on last character applies style on whole text on next open
+		// #6165 : applying style on last character applies style on whole text on next open 
+		/*if (itText->ch == SpecialChars::PARSEP && itText->parstyle != NULL)
+			itText->parstyle->charStyle().applyCharStyle(style);*/
+
+		// Does not work well, do not reenable before checking #9337, #9376 and #9428
 		// #9173 et. al.: move charstyle to parstyle if whole paragraph is affected
-		if (itText->ch == SpecialChars::PARSEP && itText->parstyle != NULL && lastParStart >= 0)
+		/*if (itText->ch == SpecialChars::PARSEP && itText->parstyle != NULL && lastParStart >= 0)
 		{
 			eraseCharStyle(lastParStart, i - lastParStart, style);
 			itText->parstyle->charStyle().applyCharStyle(style);
 			lastParStart = i + 1;
-		}
+		}*/
 		itText->applyCharStyle(style);
 	}
-	if (pos + signed(len) == length() && lastParStart >= 0)
+	// Does not work well,  do not reenable before checking #9337, #9376 and #9428
+	/*if (pos + signed(len) == length() && lastParStart >= 0)
 	{
 		eraseCharStyle(lastParStart, length() - lastParStart, style);
 		d->trailingStyle.charStyle().applyCharStyle(style);
-	}
+	}*/
 	
 	invalidate(pos, pos + len);
 }
@@ -674,10 +679,11 @@ void StoryText::eraseCharStyle(int pos, uint len, const CharStyle& style )
 			itText->parstyle->charStyle().eraseCharStyle(style);
 		itText->eraseCharStyle(style);
 	}
-	if (pos + signed(len) == length())
+	// Does not work well,  do not reenable before checking #9337, #9376 and #9428
+	/*if (pos + signed(len) == length())
 	{
 		d->trailingStyle.charStyle().eraseCharStyle(style);
-	}
+	}*/
 	
 	invalidate(pos, pos + len);
 }
@@ -1298,7 +1304,7 @@ int StoryText::screenToPosition(FPoint coord) const
 		if (xpos + 1.0 > coord.x()) // allow 1pt after end of line
 			return ls.lastItem + 1;
 		else if (coord.x() <= ls.x + ls.width) // last line of paragraph?
-			return ls.lastItem;
+			return ((ls.lastItem == lastFrameItem) ? (ls.lastItem + 1) : ls.lastItem);
 		else if (xpos < ls.x + 0.01 && maxx >= coord.x()) // check for empty line
 			return ls.firstItem;
 	}
